@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.5.  */
+/* A Bison parser, made by GNU Bison 3.4.1.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +31,9 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
+
 #ifndef YY_YY_CONFIGPARSER_H_INCLUDED
 # define YY_YY_CONFIGPARSER_H_INCLUDED
 /* Debug traces.  */
@@ -104,48 +108,55 @@ extern int yydebug;
     VAR_TLS_SERVICE_PEM = 314,
     VAR_TLS_SERVICE_OCSP = 315,
     VAR_TLS_PORT = 316,
-    VAR_DNSTAP = 317,
-    VAR_DNSTAP_ENABLE = 318,
-    VAR_DNSTAP_SOCKET_PATH = 319,
-    VAR_DNSTAP_SEND_IDENTITY = 320,
-    VAR_DNSTAP_SEND_VERSION = 321,
-    VAR_DNSTAP_IDENTITY = 322,
-    VAR_DNSTAP_VERSION = 323,
-    VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES = 324,
-    VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES = 325,
-    VAR_REMOTE_CONTROL = 326,
-    VAR_CONTROL_ENABLE = 327,
-    VAR_CONTROL_INTERFACE = 328,
-    VAR_CONTROL_PORT = 329,
-    VAR_SERVER_KEY_FILE = 330,
-    VAR_SERVER_CERT_FILE = 331,
-    VAR_CONTROL_KEY_FILE = 332,
-    VAR_CONTROL_CERT_FILE = 333,
-    VAR_KEY = 334,
-    VAR_ALGORITHM = 335,
-    VAR_SECRET = 336,
-    VAR_PATTERN = 337,
-    VAR_NAME = 338,
-    VAR_ZONEFILE = 339,
-    VAR_NOTIFY = 340,
-    VAR_PROVIDE_XFR = 341,
-    VAR_AXFR = 342,
-    VAR_UDP = 343,
-    VAR_NOTIFY_RETRY = 344,
-    VAR_ALLOW_NOTIFY = 345,
-    VAR_REQUEST_XFR = 346,
-    VAR_ALLOW_AXFR_FALLBACK = 347,
-    VAR_OUTGOING_INTERFACE = 348,
-    VAR_MAX_REFRESH_TIME = 349,
-    VAR_MIN_REFRESH_TIME = 350,
-    VAR_MAX_RETRY_TIME = 351,
-    VAR_MIN_RETRY_TIME = 352,
-    VAR_MULTI_MASTER_CHECK = 353,
-    VAR_SIZE_LIMIT_XFR = 354,
-    VAR_ZONESTATS = 355,
-    VAR_INCLUDE_PATTERN = 356,
-    VAR_ZONE = 357,
-    VAR_RRL_WHITELIST = 358
+    VAR_CPU_AFFINITY = 317,
+    VAR_XFRD_CPU_AFFINITY = 318,
+    VAR_SERVER_CPU_AFFINITY = 319,
+    VAR_DROP_UPDATES = 320,
+    VAR_DNSTAP = 321,
+    VAR_DNSTAP_ENABLE = 322,
+    VAR_DNSTAP_SOCKET_PATH = 323,
+    VAR_DNSTAP_SEND_IDENTITY = 324,
+    VAR_DNSTAP_SEND_VERSION = 325,
+    VAR_DNSTAP_IDENTITY = 326,
+    VAR_DNSTAP_VERSION = 327,
+    VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES = 328,
+    VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES = 329,
+    VAR_REMOTE_CONTROL = 330,
+    VAR_CONTROL_ENABLE = 331,
+    VAR_CONTROL_INTERFACE = 332,
+    VAR_CONTROL_PORT = 333,
+    VAR_SERVER_KEY_FILE = 334,
+    VAR_SERVER_CERT_FILE = 335,
+    VAR_CONTROL_KEY_FILE = 336,
+    VAR_CONTROL_CERT_FILE = 337,
+    VAR_KEY = 338,
+    VAR_ALGORITHM = 339,
+    VAR_SECRET = 340,
+    VAR_PATTERN = 341,
+    VAR_NAME = 342,
+    VAR_ZONEFILE = 343,
+    VAR_NOTIFY = 344,
+    VAR_PROVIDE_XFR = 345,
+    VAR_AXFR = 346,
+    VAR_UDP = 347,
+    VAR_NOTIFY_RETRY = 348,
+    VAR_ALLOW_NOTIFY = 349,
+    VAR_REQUEST_XFR = 350,
+    VAR_ALLOW_AXFR_FALLBACK = 351,
+    VAR_OUTGOING_INTERFACE = 352,
+    VAR_MAX_REFRESH_TIME = 353,
+    VAR_MIN_REFRESH_TIME = 354,
+    VAR_MAX_RETRY_TIME = 355,
+    VAR_MIN_RETRY_TIME = 356,
+    VAR_MULTI_MASTER_CHECK = 357,
+    VAR_SIZE_LIMIT_XFR = 358,
+    VAR_ZONESTATS = 359,
+    VAR_INCLUDE_PATTERN = 360,
+    VAR_ZONE = 361,
+    VAR_RRL_WHITELIST = 362,
+    VAR_SERVERS = 363,
+    VAR_BINDTODEVICE = 364,
+    VAR_SETFIB = 365
   };
 #endif
 /* Tokens.  */
@@ -208,64 +219,72 @@ extern int yydebug;
 #define VAR_TLS_SERVICE_PEM 314
 #define VAR_TLS_SERVICE_OCSP 315
 #define VAR_TLS_PORT 316
-#define VAR_DNSTAP 317
-#define VAR_DNSTAP_ENABLE 318
-#define VAR_DNSTAP_SOCKET_PATH 319
-#define VAR_DNSTAP_SEND_IDENTITY 320
-#define VAR_DNSTAP_SEND_VERSION 321
-#define VAR_DNSTAP_IDENTITY 322
-#define VAR_DNSTAP_VERSION 323
-#define VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES 324
-#define VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES 325
-#define VAR_REMOTE_CONTROL 326
-#define VAR_CONTROL_ENABLE 327
-#define VAR_CONTROL_INTERFACE 328
-#define VAR_CONTROL_PORT 329
-#define VAR_SERVER_KEY_FILE 330
-#define VAR_SERVER_CERT_FILE 331
-#define VAR_CONTROL_KEY_FILE 332
-#define VAR_CONTROL_CERT_FILE 333
-#define VAR_KEY 334
-#define VAR_ALGORITHM 335
-#define VAR_SECRET 336
-#define VAR_PATTERN 337
-#define VAR_NAME 338
-#define VAR_ZONEFILE 339
-#define VAR_NOTIFY 340
-#define VAR_PROVIDE_XFR 341
-#define VAR_AXFR 342
-#define VAR_UDP 343
-#define VAR_NOTIFY_RETRY 344
-#define VAR_ALLOW_NOTIFY 345
-#define VAR_REQUEST_XFR 346
-#define VAR_ALLOW_AXFR_FALLBACK 347
-#define VAR_OUTGOING_INTERFACE 348
-#define VAR_MAX_REFRESH_TIME 349
-#define VAR_MIN_REFRESH_TIME 350
-#define VAR_MAX_RETRY_TIME 351
-#define VAR_MIN_RETRY_TIME 352
-#define VAR_MULTI_MASTER_CHECK 353
-#define VAR_SIZE_LIMIT_XFR 354
-#define VAR_ZONESTATS 355
-#define VAR_INCLUDE_PATTERN 356
-#define VAR_ZONE 357
-#define VAR_RRL_WHITELIST 358
+#define VAR_CPU_AFFINITY 317
+#define VAR_XFRD_CPU_AFFINITY 318
+#define VAR_SERVER_CPU_AFFINITY 319
+#define VAR_DROP_UPDATES 320
+#define VAR_DNSTAP 321
+#define VAR_DNSTAP_ENABLE 322
+#define VAR_DNSTAP_SOCKET_PATH 323
+#define VAR_DNSTAP_SEND_IDENTITY 324
+#define VAR_DNSTAP_SEND_VERSION 325
+#define VAR_DNSTAP_IDENTITY 326
+#define VAR_DNSTAP_VERSION 327
+#define VAR_DNSTAP_LOG_AUTH_QUERY_MESSAGES 328
+#define VAR_DNSTAP_LOG_AUTH_RESPONSE_MESSAGES 329
+#define VAR_REMOTE_CONTROL 330
+#define VAR_CONTROL_ENABLE 331
+#define VAR_CONTROL_INTERFACE 332
+#define VAR_CONTROL_PORT 333
+#define VAR_SERVER_KEY_FILE 334
+#define VAR_SERVER_CERT_FILE 335
+#define VAR_CONTROL_KEY_FILE 336
+#define VAR_CONTROL_CERT_FILE 337
+#define VAR_KEY 338
+#define VAR_ALGORITHM 339
+#define VAR_SECRET 340
+#define VAR_PATTERN 341
+#define VAR_NAME 342
+#define VAR_ZONEFILE 343
+#define VAR_NOTIFY 344
+#define VAR_PROVIDE_XFR 345
+#define VAR_AXFR 346
+#define VAR_UDP 347
+#define VAR_NOTIFY_RETRY 348
+#define VAR_ALLOW_NOTIFY 349
+#define VAR_REQUEST_XFR 350
+#define VAR_ALLOW_AXFR_FALLBACK 351
+#define VAR_OUTGOING_INTERFACE 352
+#define VAR_MAX_REFRESH_TIME 353
+#define VAR_MIN_REFRESH_TIME 354
+#define VAR_MAX_RETRY_TIME 355
+#define VAR_MIN_RETRY_TIME 356
+#define VAR_MULTI_MASTER_CHECK 357
+#define VAR_SIZE_LIMIT_XFR 358
+#define VAR_ZONESTATS 359
+#define VAR_INCLUDE_PATTERN 360
+#define VAR_ZONE 361
+#define VAR_RRL_WHITELIST 362
+#define VAR_SERVERS 363
+#define VAR_BINDTODEVICE 364
+#define VAR_SETFIB 365
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-
 union YYSTYPE
 {
-#line 37 "configparser.y" /* yacc.c:1910  */
+#line 40 "configparser.y"
 
   char *str;
   long long llng;
   int bln;
   struct ip_address_option *ip;
+  struct range_option *range;
+  struct cpu_option *cpu;
+
+#line 286 "configparser.h"
 
-#line 267 "configparser.h" /* yacc.c:1910  */
 };
-
 typedef union YYSTYPE YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define YYSTYPE_IS_DECLARED 1
