@@ -11,12 +11,14 @@
 #define	_NSD_H_
 
 #include <signal.h>
+#include <net/if.h>
 #ifdef HAVE_OPENSSL_SSL_H
 #include <openssl/ssl.h>
 #endif
 
 #include "dns.h"
 #include "edns.h"
+#include "bitset.h"
 struct netio_handler;
 struct nsd_options;
 struct udb_base;
@@ -111,6 +113,7 @@ typedef	unsigned long stc_type;
 #endif /* USE_ZONE_STATS */
 
 #define NSD_SOCKET_IS_OPTIONAL (1<<0)
+#define NSD_BIND_DEVICE (1<<1)
 
 struct nsd_addrinfo
 {
@@ -123,14 +126,22 @@ struct nsd_addrinfo
 
 struct nsd_socket
 {
-	struct nsd_addrinfo	addr;
-	int			s;
-	int			flags;
+	struct nsd_addrinfo addr;
+	int s;
+	int flags;
+	struct nsd_bitset *servers;
+	char device[IFNAMSIZ];
+	int fib;
 };
 
 struct nsd_child
 {
-	 /* The type of child process (UDP or TCP handler). */
+#ifdef HAVE_CPUSET_T
+	/* Processor(s) that child process must run on (if applicable). */
+	cpuset_t *cpuset;
+#endif
+
+	/* The type of child process (UDP or TCP handler). */
 	int kind;
 
 	/* The child's process id.  */
@@ -223,6 +234,12 @@ struct	nsd
 	unsigned char		*nsid;
 	uint8_t 		file_rotation_ok;
 
+#ifdef HAVE_CPUSET_T
+	int			use_cpu_affinity;
+	cpuset_t*		cpuset;
+	cpuset_t*		xfrd_cpuset;
+#endif
+
 	/* number of interfaces */
 	size_t	ifs;
 	/* non0 if so_reuseport is in use, if so, tcp, udp array increased */
