@@ -20,6 +20,9 @@
 #include <syslog.h>
 #endif /* HAVE_SYSLOG_H */
 #include <unistd.h>
+#ifdef HAVE_SYS_RANDOM_H
+#include <sys/random.h>
+#endif
 
 #include "util.h"
 #include "region-allocator.h"
@@ -923,8 +926,15 @@ compare_serial(uint32_t a, uint32_t b)
 uint16_t
 qid_generate(void)
 {
+#ifdef HAVE_GETRANDOM
+	uint16_t r;
+	if(getrandom(&r, sizeof(r), 0) == -1) {
+		log_msg(LOG_ERR, "getrandom failed: %s", strerror(errno));
+		exit(1);
+	}
+	return r;
+#elif defined(HAVE_ARC4RANDOM)
     /* arc4random_uniform not needed because range is a power of 2 */
-#ifdef HAVE_ARC4RANDOM
     return (uint16_t) arc4random();
 #else
     return (uint16_t) random();
@@ -934,9 +944,16 @@ qid_generate(void)
 int
 random_generate(int max)
 {
-#ifdef HAVE_ARC4RANDOM_UNIFORM
+#ifdef HAVE_GETRANDOM
+	int r;
+	if(getrandom(&r, sizeof(r), 0) == -1) {
+		log_msg(LOG_ERR, "getrandom failed: %s", strerror(errno));
+		exit(1);
+	}
+	return (int)(((unsigned)r)%max);
+#elif defined(HAVE_ARC4RANDOM_UNIFORM)
     return (int) arc4random_uniform(max);
-#elif HAVE_ARC4RANDOM
+#elif defined(HAVE_ARC4RANDOM)
     return (int) (arc4random() % max);
 #else
     return (int) ((unsigned)random() % max);
@@ -1086,6 +1103,35 @@ addr2str(
 		strlcpy(str, "[unknown ip4, inet_ntop failed]", len);
 }
 
+void
+addrport2str(
+#ifdef INET6
+	struct sockaddr_storage *addr
+#else
+	struct sockaddr_in *addr
+#endif
+	, char* str, size_t len)
+{
+	char ip[256];
+#ifdef INET6
+	if (addr->ss_family == AF_INET6) {
+		if (!inet_ntop(AF_INET6,
+			&((struct sockaddr_in6 *)addr)->sin6_addr, ip, sizeof(ip)))
+			strlcpy(ip, "[unknown ip6, inet_ntop failed]", sizeof(ip));
+		/* append port number */
+		snprintf(str, len, "%s@%u", ip,
+			(unsigned)ntohs(((struct sockaddr_in6 *)addr)->sin6_port));
+		return;
+	} else
+#endif
+	if (!inet_ntop(AF_INET, &((struct sockaddr_in *)addr)->sin_addr,
+		ip, sizeof(ip)))
+		strlcpy(ip, "[unknown ip4, inet_ntop failed]", sizeof(ip));
+	/* append port number */
+	snprintf(str, len, "%s@%u", ip,
+		(unsigned)ntohs(((struct sockaddr_in *)addr)->sin_port));
+}
+
 void
 append_trailing_slash(const char** dirname, region_type* region)
 {
@@ -1120,3 +1166,26 @@ error(const char *format, ...)
 	exit(1);
 }
 
+#ifdef HAVE_CPUSET_T
+#if __linux__ || __FreeBSD__
+int number_of_cpus(void)
+{
+	return (int)sysconf(_SC_NPROCESSORS_CONF);
+}
+#endif
+#if __linux__
+int set_cpu_affinity(cpuset_t *set)
+{
+	assert(set != NULL);
+	return sched_setaffinity(getpid(), sizeof(*set), set);
+}
+#endif
+#if __FreeBSD__
+int set_cpu_affinity(cpuset_t *set)
+{
+	assert(set != NULL);
+	return cpuset_setaffinity(
+		CPU_LEVEL_WHICH, CPU_WHICH_PID, -1, sizeof(*set), set);
+}
+#endif
+#endif /* HAVE_CPUSET_T */
